'''
@author: vmmhypervisor
'''

import math
#from fnmatch import filter

import golib.common as common


MIN = "min"
MAX = "max"
MEAN = "mean"
RANGE = "range"
STANDARD_DEVIATION = "sd"
COEFFICIENT_VARIATION = "cv"
RSD = "%rsd"		# Relative Standard Deviation

FIRST_PERCENTILE = "1st_%ile"
FIFTH_PERCENTILE = "5th_%ile"
FIRST_QUARTILE = "1st_qu"
MEDIAN = "median"
THIRD_QUARTILE = "3rd_qu"
NINETY_FIFTH_PERCENTILE = "95th_%ile"
NINETY_NINETH_PERCENTILE = "99th_%ile"
QUARTILE_DEVIATION = "qd"
CQD = "cqd"		# Coefficient of Quartile Deviation
TRIMED_MEAN = "trimed_mean"

SUM = "sum"
DATA_SIZE = "num_data"
INVALID_DATA_SIZE = "num_invalid_data"

SKEWNESS = "skew"  # Pearson's median skewness coefficient
KURTOSIS = "kurt"

STATUS = "status"
STATUS_INVALID = -1
STATUS_NORMAL = 0
STATUS_ALL_ZERO = 1

BASIC_STATISTICS_LIST = (
    MIN,
    MEAN,
    MAX,
    STANDARD_DEVIATION,
    RSD,
)

VERBOSE_STATISTICS_LIST = [
    MEAN,
    MIN,
    FIRST_PERCENTILE,
    FIFTH_PERCENTILE,
    FIRST_QUARTILE,
    MEDIAN,
    THIRD_QUARTILE,
    NINETY_FIFTH_PERCENTILE,
    NINETY_NINETH_PERCENTILE,
    MAX,
    TRIMED_MEAN,
    DATA_SIZE,
    RANGE,
    STANDARD_DEVIATION,
    COEFFICIENT_VARIATION,
    RSD,
    QUARTILE_DEVIATION,
    CQD,
    SKEWNESS,
    KURTOSIS,
]

DATABASE_STATISTICS_LIST	= VERBOSE_STATISTICS_LIST \
    + [INVALID_DATA_SIZE, SUM, STATUS]


def getStatisticsFieldList(verbose):
    statistics_list = BASIC_STATISTICS_LIST
    if verbose:
        statistics_list = VERBOSE_STATISTICS_LIST

    return list(statistics_list)


class Calculator:
    nan = float("nan")
    __null_return_value = {
        MEAN: nan,
        RANGE: nan,
        MIN: nan,
        FIRST_PERCENTILE: nan,
        FIFTH_PERCENTILE: nan,
        FIRST_QUARTILE: nan,
        MEDIAN: nan,
        THIRD_QUARTILE: nan,
        NINETY_FIFTH_PERCENTILE: nan,
        NINETY_NINETH_PERCENTILE: nan,
        MAX: nan,
        QUARTILE_DEVIATION: nan,
        CQD: nan,
        TRIMED_MEAN: nan,
        STANDARD_DEVIATION: nan,
        COEFFICIENT_VARIATION: nan,
        RSD: nan,
        SUM: nan,
        SKEWNESS: nan,
        KURTOSIS: nan,
        DATA_SIZE: 0,
        INVALID_DATA_SIZE: 0,
        STATUS: STATUS_INVALID,
    }

    __TRIM_RATE = 0.05

    @classmethod
    def __isValidData(cls, data):
        return common.isFloat(data) and not common.isNaN(data)

    @classmethod
    def __getPercentile(cls, percentile_index, data_list, max_data_index):
        #coef_lhs	= 1.0 - (percentile_index - int(percentile_index))
        #coef_rhs	= 1.0 - coef_lhs

        lhs_index = max(int(percentile_index), 0)
        rhs_index = min(int(math.ceil(percentile_index)), max_data_index)

        return (data_list[lhs_index] + data_list[rhs_index]) * 0.5
        # return data_list[lhs_index] * coef_lhs + data_list[rhs_index] *
        # coef_rhs

    @classmethod
    def calcTrimedMean(cls, sorted_data_list, trim):
        data_size = len(sorted_data_list)
        if data_size < 4:
            return common.safeDivision(sum(sorted_data_list), len(sorted_data_list))

        begin_index = int(data_size * trim)
        end_index = int(math.ceil(data_size * (1.0 - trim)))
        trimed_data_list = sorted_data_list[begin_index: end_index]

        return common.safeDivision(sum(trimed_data_list), len(trimed_data_list))

    @classmethod
    def getMean(cls, data_list):
        valid_data_list = filter(cls.__isValidData, data_list)
        valid_data_list = map(common.convertValue, valid_data_list)

        return common.safeDivision(sum(valid_data_list), len(valid_data_list))

    @classmethod
    def getStatistics(cls, data_list):
        if common.isEmptyList(data_list):
            return cls.__null_return_value

        valid_data_list = filter(cls.__isValidData, data_list)
        valid_data_list = map(common.convertValue, valid_data_list)
        data_size = len(valid_data_list)
        if data_size == 0:
            #logging.debug("list length is zero" + str(logging.getLogger().findCaller()))
            return cls.__null_return_value

        sum_value = sum(valid_data_list)
        mean = sum_value / float(data_size)

        """
        gm = valid_data_list[0]
        for x in valid_data_list[1:]:
            gm *= x
        #print (1.0 / float(data_size))
        gm = math.pow(gm, (1.0 / float(data_size)))
        if gm == float("inf"):
            print "##", data_size, valid_data_list
        #"""

        sigma_square = 0.0
        sigma_cube = 0.0
        sigma_quad = 0.0
        variance = 0.0
        for data in valid_data_list:
            variance += (data - mean) ** 2

            """
            square	= data ** 2
            sigma_square	+= square
            cube	= square * data
            sigma_cube	+= cube
            quad	= cube * data
            sigma_quad	+= quad
            """

        variance = common.safeDivision(variance, data_size - 1)
        sd = math.sqrt(variance)
        cv = common.safeDivision(sd, mean)
        rsd = cv * 100.0

        sorted_data_list = sorted(valid_data_list)

        max_quartile_index = data_size - 1
        percentile_index_base = max_quartile_index / 100.0

        first_percentile = cls.__getPercentile(
            percentile_index_base * 1, sorted_data_list, max_quartile_index)
        fifth_percentile = cls.__getPercentile(
            percentile_index_base * 5, sorted_data_list, max_quartile_index)
        ninety_fifth_percentile = cls.__getPercentile(
            percentile_index_base * 95, sorted_data_list, max_quartile_index)
        ninety_nineth_percentile = cls.__getPercentile(
            percentile_index_base * 99, sorted_data_list, max_quartile_index)

        lower_quartile = cls.__getPercentile(
            percentile_index_base * 25, sorted_data_list, max_quartile_index)
        median = cls.__getPercentile(
            percentile_index_base * 50, sorted_data_list, max_quartile_index)
        upper_quartile = cls.__getPercentile(
            percentile_index_base * 75, sorted_data_list, max_quartile_index)
        quartile_deviation = (upper_quartile - lower_quartile) * 0.5
        cqd = common.safeDivision(
            2.0 * quartile_deviation, upper_quartile + lower_quartile)

        skewness = common.safeDivision(3.0 * (mean - median), sd)
        kurtosis = float("nan")
        """
        n	= float(data_size)
        tmp	= sigma_cube / n - (3.0 * mean * sigma_square) / n + 2 * mean ** 3
        skewness	= common.safeDivision(tmp, sd ** 3)
        
        tmp	= sigma_quad / n - 4.0 * mean * sigma_cube / n + (6.0 * mean ** 2 * sigma_square) / n - 3.0 * mean ** 4
        kurtosis	= common.safeDivision(tmp, sd ** 4)
        """

        min_value = sorted_data_list[0]
        max_value = sorted_data_list[-1]
        status = STATUS_NORMAL
        if min_value == 0 and max_value == 0:
            status = STATUS_ALL_ZERO

        return {
            MIN: min_value,
            MEAN: mean,
            MAX: max_value,
            RANGE: max_value - min_value,
            FIRST_PERCENTILE: first_percentile,
            FIFTH_PERCENTILE: fifth_percentile,
            FIRST_QUARTILE: lower_quartile,
            MEDIAN: median,
            THIRD_QUARTILE: upper_quartile,
            NINETY_FIFTH_PERCENTILE: ninety_fifth_percentile,
            NINETY_NINETH_PERCENTILE: ninety_nineth_percentile,
            QUARTILE_DEVIATION: quartile_deviation,
            CQD: cqd,
            TRIMED_MEAN: cls.calcTrimedMean(sorted_data_list, cls.__TRIM_RATE),
            STANDARD_DEVIATION: sd,
            COEFFICIENT_VARIATION: cv,
            RSD: rsd,
            SUM: sum_value,
            SKEWNESS: skewness,
            KURTOSIS: kurtosis,
            DATA_SIZE: data_size,
            INVALID_DATA_SIZE: len(data_list) - data_size,
            STATUS: status
        }
